@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-close-confirmation-dialog.c,v 1.14 2006/03/14 10:53:12 pborelli Exp $
+ * $Id: gedit-close-confirmation-dialog.c,v 1.20 2006/08/24 15:08:34 paolo Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,9 +35,11 @@
 #include <glib/gi18n.h>
 
 #include "gedit-close-confirmation-dialog.h"
+#include <gedit/gedit-app.h>
 #include <gedit/gedit-utils.h>
 #include <gedit/gedit-window.h>
 
+
 /* Properties */
 enum 
 {
@@ -62,8 +64,6 @@ enum
 	N_COLUMNS
 };
 
-typedef struct _GeditCloseConfirmationDialogPrivate GeditCloseConfirmationDialogPrivate;
-
 struct _GeditCloseConfirmationDialogPrivate 
 {
 	gboolean     logout_mode;
@@ -73,11 +73,17 @@ struct _GeditCloseConfirmationDialogPrivate
 	GList       *selected_documents;
 
 	GtkTreeModel *list_store;
+	
+	gboolean     disable_save_to_disk;
 };
 
-#define GEDIT_CLOSE_CONFIRMATION_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEDIT_TYPE_CLOSE_CONFIRMATION_DIALOG, GeditCloseConfirmationDialogPrivate))
+#define GEDIT_CLOSE_CONFIRMATION_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), \
+							GEDIT_TYPE_CLOSE_CONFIRMATION_DIALOG, \
+							GeditCloseConfirmationDialogPrivate))
 
-#define GET_MODE(priv) (((priv->unsaved_documents != NULL) && (priv->unsaved_documents->next == NULL)) ? SINGLE_DOC_MODE : MULTIPLE_DOCS_MODE)
+#define GET_MODE(priv) (((priv->unsaved_documents != NULL) && \
+			 (priv->unsaved_documents->next == NULL)) ? \
+			  SINGLE_DOC_MODE : MULTIPLE_DOCS_MODE)
 
 G_DEFINE_TYPE(GeditCloseConfirmationDialog, gedit_close_confirmation_dialog, GTK_TYPE_DIALOG)
 
@@ -98,20 +104,22 @@ response_cb (GeditCloseConfirmationDialog *dlg,
 
 	g_return_if_fail (GEDIT_IS_CLOSE_CONFIRMATION_DIALOG (dlg));
 
-	priv = GEDIT_CLOSE_CONFIRMATION_DIALOG_GET_PRIVATE (dlg);
+	priv = dlg->priv;
 	
 	if (priv->selected_documents != NULL)
 		g_list_free (priv->selected_documents);
-	
+
 	if (response_id == GTK_RESPONSE_YES)
 	{
 		if (GET_MODE (priv) == SINGLE_DOC_MODE)
+		{
 			priv->selected_documents = 
 				g_list_copy (priv->unsaved_documents);
+		}
 		else
 		{
 			g_return_if_fail (priv->list_store);
-				
+
 			priv->selected_documents =
 				get_selected_docs (priv->list_store);
 		}
@@ -124,11 +132,7 @@ static void
 set_logout_mode (GeditCloseConfirmationDialog *dlg,
 		 gboolean                      logout_mode)
 {
-	GeditCloseConfirmationDialogPrivate *priv;
-	
-	priv = GEDIT_CLOSE_CONFIRMATION_DIALOG_GET_PRIVATE (dlg);
-
-	priv->logout_mode = logout_mode;
+	dlg->priv->logout_mode = logout_mode;
 	
 	if (logout_mode)
 	{
@@ -151,25 +155,52 @@ set_logout_mode (GeditCloseConfirmationDialog *dlg,
 				       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 	}
 	
-	gtk_dialog_add_button (GTK_DIALOG (dlg),
-			       GTK_STOCK_SAVE, GTK_RESPONSE_YES);
+	if (dlg->priv->disable_save_to_disk)
+	{
+		gtk_dialog_set_default_response	(GTK_DIALOG (dlg),
+						 GTK_RESPONSE_NO);
+	}
+	else
+	{
+		const gchar *stock_id = GTK_STOCK_SAVE;
+		
+		if (GET_MODE (dlg->priv) == SINGLE_DOC_MODE)
+		{
+			GeditDocument *doc;
+			
+			doc = GEDIT_DOCUMENT (dlg->priv->unsaved_documents->data);
+			
+			if (gedit_document_get_readonly (doc) || 
+			    gedit_document_is_untitled (doc))
+				stock_id = GTK_STOCK_SAVE_AS;
+		}
 
-	gtk_dialog_set_default_response	(GTK_DIALOG (dlg), GTK_RESPONSE_YES);
+		gtk_dialog_add_button (GTK_DIALOG (dlg),
+				       stock_id, 
+				       GTK_RESPONSE_YES);
+
+		gtk_dialog_set_default_response	(GTK_DIALOG (dlg), 
+						 GTK_RESPONSE_YES);
+	}
 }
 
 static void 
 gedit_close_confirmation_dialog_init (GeditCloseConfirmationDialog *dlg)
 {
-	GeditCloseConfirmationDialogPrivate *priv;
 	AtkObject *atk_obj;
 
-	priv = GEDIT_CLOSE_CONFIRMATION_DIALOG_GET_PRIVATE (dlg);
+	dlg->priv = GEDIT_CLOSE_CONFIRMATION_DIALOG_GET_PRIVATE (dlg);
+
+	dlg->priv->disable_save_to_disk = 
+			gedit_app_get_lockdown (gedit_app_get_default ()) 
+			& GEDIT_LOCKDOWN_SAVE_TO_DISK;
 
 	gtk_container_set_border_width (GTK_CONTAINER (dlg), 5);		
 	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dlg)->vbox), 14);
 	gtk_window_set_resizable (GTK_WINDOW (dlg), FALSE);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
-
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dlg), TRUE);
+	
 	gtk_window_set_title (GTK_WINDOW (dlg), "");
 
 	gtk_window_set_modal (GTK_WINDOW (dlg), TRUE);
@@ -190,8 +221,8 @@ gedit_close_confirmation_dialog_finalize (GObject *object)
 {
 	GeditCloseConfirmationDialogPrivate *priv;
 
-	priv = GEDIT_CLOSE_CONFIRMATION_DIALOG_GET_PRIVATE (object);
-	
+	priv = GEDIT_CLOSE_CONFIRMATION_DIALOG (object)->priv;
+
 	if (priv->unsaved_documents != NULL)
 		g_list_free (priv->unsaved_documents);
 
@@ -209,10 +240,8 @@ gedit_close_confirmation_dialog_set_property (GObject      *object,
 					      GParamSpec   *pspec)
 {
 	GeditCloseConfirmationDialog *dlg;
-	GeditCloseConfirmationDialogPrivate *priv;
 
 	dlg = GEDIT_CLOSE_CONFIRMATION_DIALOG (object);
-	priv = GEDIT_CLOSE_CONFIRMATION_DIALOG_GET_PRIVATE (object);
 
 	switch (prop_id)
 	{
@@ -238,7 +267,7 @@ gedit_close_confirmation_dialog_get_property (GObject    *object,
 {
 	GeditCloseConfirmationDialogPrivate *priv;
 
-	priv = GEDIT_CLOSE_CONFIRMATION_DIALOG_GET_PRIVATE (object);
+	priv = GEDIT_CLOSE_CONFIRMATION_DIALOG (object)->priv;
 
 	switch( prop_id )
 	{
@@ -249,7 +278,7 @@ gedit_close_confirmation_dialog_get_property (GObject    *object,
 		case PROP_LOGOUT_MODE:
 			g_value_set_boolean (value, priv->logout_mode);
 			break;
-			
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -274,7 +303,7 @@ gedit_close_confirmation_dialog_class_init (GeditCloseConfirmationDialogClass *k
 							       "List of Unsaved Documents",
 							       (G_PARAM_READWRITE | 
 							        G_PARAM_CONSTRUCT_ONLY)));
-							        
+
 	g_object_class_install_property (gobject_class,
 					 PROP_LOGOUT_MODE,
 					 g_param_spec_boolean ("logout_mode",
@@ -318,13 +347,9 @@ get_selected_docs (GtkTreeModel *store)
 GList *
 gedit_close_confirmation_dialog_get_selected_documents (GeditCloseConfirmationDialog *dlg)
 {
-	GeditCloseConfirmationDialogPrivate *priv;
-
 	g_return_val_if_fail (GEDIT_IS_CLOSE_CONFIRMATION_DIALOG (dlg), NULL);
 
-	priv = GEDIT_CLOSE_CONFIRMATION_DIALOG_GET_PRIVATE (dlg);
-
-	return g_list_copy (priv->selected_documents);
+	return g_list_copy (dlg->priv->selected_documents);
 }
 
 GtkWidget *
@@ -468,12 +493,8 @@ build_single_doc_dialog (GeditCloseConfirmationDialog *dlg)
 	gchar         *str;
 	gchar         *markup_str;
 
-	GeditCloseConfirmationDialogPrivate *priv;
-
-	priv = GEDIT_CLOSE_CONFIRMATION_DIALOG_GET_PRIVATE (dlg);
-
-	g_return_if_fail (priv->unsaved_documents->data != NULL);
-	doc = GEDIT_DOCUMENT (priv->unsaved_documents->data);
+	g_return_if_fail (dlg->priv->unsaved_documents->data != NULL);
+	doc = GEDIT_DOCUMENT (dlg->priv->unsaved_documents->data);
 
 	/* Image */
 	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, 
@@ -488,8 +509,14 @@ build_single_doc_dialog (GeditCloseConfirmationDialog *dlg)
 	gtk_label_set_selectable (GTK_LABEL (primary_label), TRUE);
 
 	doc_name = gedit_document_get_short_name_for_display (doc);
-	str = g_strdup_printf (_("Save the changes to document \"%s\" before closing?"),
-			       doc_name);
+
+	if (dlg->priv->disable_save_to_disk)
+		str = g_strdup_printf (_("Changes to document \"%s\" will be permanently lost."),
+				       doc_name);
+	else
+		str = g_strdup_printf (_("Save the changes to document \"%s\" before closing?"),
+				       doc_name);
+
 	g_free (doc_name);
 
 	markup_str = g_strconcat ("<span weight=\"bold\" size=\"larger\">", str, "</span>", NULL);
@@ -499,7 +526,10 @@ build_single_doc_dialog (GeditCloseConfirmationDialog *dlg)
 	g_free (markup_str);
 
 	/* Secondary label */
-	str = get_text_secondary_label (doc);
+	if (dlg->priv->disable_save_to_disk)
+		str = g_strdup (_("Saving has been disabled by the system administrator."));	
+	else
+		str = get_text_secondary_label (doc);
 	secondary_label = gtk_label_new (str);
 	g_free (str);
 	gtk_label_set_line_wrap (GTK_LABEL (secondary_label), TRUE);
@@ -597,16 +627,19 @@ create_treeview (GeditCloseConfirmationDialogPrivate *priv)
 	priv->list_store = GTK_TREE_MODEL (store);
 	
 	/* Add columns */
-	renderer = gtk_cell_renderer_toggle_new ();
-	g_signal_connect (renderer, "toggled",
-			  G_CALLBACK (save_toggled), store);
+	if (!priv->disable_save_to_disk)
+	{
+		renderer = gtk_cell_renderer_toggle_new ();
+		g_signal_connect (renderer, "toggled",
+				  G_CALLBACK (save_toggled), store);
 
-	column = gtk_tree_view_column_new_with_attributes ("Save?",
-							   renderer,
-							   "active",
-							   SAVE_COLUMN,
-							   NULL);
-	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
+		column = gtk_tree_view_column_new_with_attributes ("Save?",
+								   renderer,
+								   "active",
+								   SAVE_COLUMN,
+								   NULL);
+		gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
+	}
 
 	renderer = gtk_cell_renderer_text_new ();
 	column = gtk_tree_view_column_new_with_attributes ("Name",
@@ -622,6 +655,7 @@ create_treeview (GeditCloseConfirmationDialogPrivate *priv)
 static void
 build_multiple_docs_dialog (GeditCloseConfirmationDialog *dlg)
 {
+	GeditCloseConfirmationDialogPrivate *priv;
 	GtkWidget *hbox;
 	GtkWidget *image;
 	GtkWidget *vbox;
@@ -634,9 +668,7 @@ build_multiple_docs_dialog (GeditCloseConfirmationDialog *dlg)
 	gchar     *str;
 	gchar     *markup_str;
 
-	GeditCloseConfirmationDialogPrivate *priv;
-
-	priv = GEDIT_CLOSE_CONFIRMATION_DIALOG_GET_PRIVATE (dlg);
+	priv = dlg->priv;
 
 	hbox = gtk_hbox_new (FALSE, 12);
 	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
@@ -659,14 +691,21 @@ build_multiple_docs_dialog (GeditCloseConfirmationDialog *dlg)
 	gtk_misc_set_alignment (GTK_MISC (primary_label), 0.0, 0.5);
 	gtk_label_set_selectable (GTK_LABEL (primary_label), TRUE);
 
-	str = g_strdup_printf (
-			ngettext ("There is %d document with unsaved changes. "
-				  "Save changes before closing?",
-				  "There are %d documents with unsaved changes. "
-				  "Save changes before closing?",
-				  g_list_length (priv->unsaved_documents)),
-			g_list_length (priv->unsaved_documents));
-	
+	if (priv->disable_save_to_disk)
+		str = g_strdup_printf (
+				ngettext ("Changes to %d document will be permanently lost.",
+					  "Changes to %d documents will be permanently lost.",
+					  g_list_length (priv->unsaved_documents)),
+				g_list_length (priv->unsaved_documents));
+	else
+		str = g_strdup_printf (
+				ngettext ("There is %d document with unsaved changes. "
+					  "Save changes before closing?",
+					  "There are %d documents with unsaved changes. "
+					  "Save changes before closing?",
+					  g_list_length (priv->unsaved_documents)),
+				g_list_length (priv->unsaved_documents));
+
 	markup_str = g_strconcat ("<span weight=\"bold\" size=\"larger\">", str, "</span>", NULL);
 	g_free (str);
 	
@@ -677,7 +716,11 @@ build_multiple_docs_dialog (GeditCloseConfirmationDialog *dlg)
 	vbox2 = gtk_vbox_new (FALSE, 8);
 	gtk_box_pack_start (GTK_BOX (vbox), vbox2, FALSE, FALSE, 0);
 
-	select_label = gtk_label_new_with_mnemonic (_("S_elect the documents you want to save:"));
+	if (priv->disable_save_to_disk)
+		select_label = gtk_label_new_with_mnemonic (_("Docum_ents with unsaved changes:"));
+	else
+		select_label = gtk_label_new_with_mnemonic (_("S_elect the documents you want to save:"));
+
 	gtk_box_pack_start (GTK_BOX (vbox2), select_label, FALSE, FALSE, 0);
 	gtk_label_set_line_wrap (GTK_LABEL (select_label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (select_label), 0.0, 0.5);
@@ -694,8 +737,12 @@ build_multiple_docs_dialog (GeditCloseConfirmationDialog *dlg)
 	gtk_container_add (GTK_CONTAINER (scrolledwindow), treeview);
 
 	/* Secondary label */
-	secondary_label = gtk_label_new (_("If you don't save, "
-					   "all your changes will be permanently lost."));
+	if (priv->disable_save_to_disk)
+		secondary_label = gtk_label_new (_("Saving has been disabled by the system administrator."));
+	else
+		secondary_label = gtk_label_new (_("If you don't save, "
+						   "all your changes will be permanently lost."));
+
 	gtk_box_pack_start (GTK_BOX (vbox2), secondary_label, FALSE, FALSE, 0);
 	gtk_label_set_line_wrap (GTK_LABEL (secondary_label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (secondary_label), 0, 0.5);
@@ -711,10 +758,10 @@ set_unsaved_document (GeditCloseConfirmationDialog *dlg,
 		      const GList                  *list)
 {
 	GeditCloseConfirmationDialogPrivate *priv;
-	
+
 	g_return_if_fail (list != NULL);	
 
-	priv = GEDIT_CLOSE_CONFIRMATION_DIALOG_GET_PRIVATE (dlg);
+	priv = dlg->priv;
 	g_return_if_fail (priv->unsaved_documents == NULL);
 
 	priv->unsaved_documents = g_list_copy ((GList *)list);
@@ -732,11 +779,7 @@ set_unsaved_document (GeditCloseConfirmationDialog *dlg,
 const GList *
 gedit_close_confirmation_dialog_get_unsaved_documents (GeditCloseConfirmationDialog *dlg)
 {
-	GeditCloseConfirmationDialogPrivate *priv;
-	
 	g_return_val_if_fail (GEDIT_IS_CLOSE_CONFIRMATION_DIALOG (dlg), NULL);
 
-	priv = GEDIT_CLOSE_CONFIRMATION_DIALOG_GET_PRIVATE (dlg);
-
-	return priv->unsaved_documents;
+	return dlg->priv->unsaved_documents;
 }
